@@ -17,7 +17,7 @@ Net::Gopher::Response - Class encapsulating Gopher responses
  		my @items = $response->as_menu;
  		foreach my $item (@items) {
  			print join("::",
- 				$item->{'type'}, $item->{'text'},
+ 				$item->{'type'}, $item->{'display'},
  				$item->{'selector'}, $item->{'host'},
  				$item->{'port'}, $item->{'gopher_plus'}
  			), "\n";
@@ -25,21 +25,25 @@ Net::Gopher::Response - Class encapsulating Gopher responses
  	}
  
  	if ($response->is_blocks) {
- 		my $info_block = $response->as_info_block;
+		# when issuing item attribute information requests, use
+		# item_blocks() to retieve Net::Gopher::Response::Blocks
+		# objects, which you can call methods like as_info() and
+		# as_admin() on to parse the block values:
+ 		my %info = $response->item_blocks('INFO')->as_info;
  
  		print join("::",
- 			$info_block->{'type'}, $info_block->{'text'},
- 			$info_block->{'selector'}, $info_block->{'host'},
- 			$info_block->{'port'}, $info_block->{'gopher_plus'}
+ 			$info{'type'}, $info{'display'},
+ 			$info{'selector'}, $info{'host'},
+ 			$info{'port'}, $info{'gopher_plus'}
  		), "\n";
  
- 		my $admin_block = $response->as_admin_block;
+ 		my %admin = $response->item_blocks('ADMIN')->as_admin;
  
- 		print "Maintained by $admin_block->{'Admin'}[0] ",
- 		      "who can be emailed at $admin_block->{'Admin'}[1]\n";
+ 		print "Maintained by $admin{'Admin'}[0] ",
+ 		      "who can be emailed at $admin{'Admin'}[1]\n";
  	}
  } else {
- 	 print $response->error;
+ 	print $response->error;
  }
  ...
 
@@ -50,8 +54,8 @@ return B<Net::Gopher::Response> objects. These objects encapsulate responses
 from Gopher and Gopher+ servers.
 
 In Gopher, a response is just a series of bytes terminated by a period on a
-line by itself. In Gopher+, a response consists of a status line, (the first
-line) of which the first character is the status (success or failure; + or -),
+line by itself. In Gopher+, a response consists of a status line (the first
+line), of which the first character is the status (success or failure; + or -),
 followed by a newline (CRLF), and the content of the response. This class
 contains methods to help you manipulate both Gopher as well as Gopher+
 responses.
@@ -66,8 +70,8 @@ use 5.005;
 use strict;
 use warnings;
 use Carp;
-use Time::Local;
 use Net::Gopher::Utility qw($CRLF $NEWLINE);
+use base qw(Net::Gopher::Response::Blocks);
 
 
 
@@ -161,19 +165,21 @@ sub status { return shift->{'status'} }
 
 Both C<content()> and C<as_string()> can be used to retrieve the strings
 containing the server's response. With C<content()>, however, if the item
-requested was a text file, then escaped periods are unescaped (i.e., '..' at
-the start of a line becomes '.'). Also, if response was terminated by a period
-on a line by itself but it isn't a text file or menu, then the period on a line
-by itself will be removed from the content (though you can still check to see
-if it was period terminated using the
+requested was text, then escaped periods are unescaped (i.e., '..' at the start
+of a line becomes '.'). Also, if the response was terminated by a period on a
+line by itself but it isn't a text file or menu, then the period on a line by
+itself will be removed from the content (though you can still check to see if
+it was period terminated using the
 L<is_terminated()|Net::Gopher::Response/is_terminated()> method). This is
-because if you were requesting an image or some other non-text file (especially
-in Gopher+), odds are you don't want the newline and period at the end the
-content.
+because if you were requesting an image or some other non-text file, odds are
+you don't want the newline and period at the end the content. And finally, if
+the item was text, then line endings are converted from CRLF and CR to LF. This
+is done so you can use '\n', '.', etc., in patterns (if any of that sounded
+unusual to you, please read C<perldoc -f binmode>).
 
-Note that in Gopher+, besides the modifications listed above, C<content()> does
-not include the status line (first line) of the response (since the status line
-isn't content).
+In Gopher+, besides the modifications listed above, C<content()> does not
+include the status line (first line) of the response (since the status line
+isn't content), only everything after it.
 
 =cut
 
@@ -206,19 +212,19 @@ sub as_string { return shift->{'response'} }
 If you got a Gopher menu as your response from the server, then you can use
 this method to parse it and return its values. When called, this method will
 parse the content returned by C<content()> and return either an array (in list
-context) or a reference to an array (in scalar context) containing hash refs as
-its elements. Each hash contains the data for one menu item, and has the
-following key=value pairs:
+context) or a reference to an array (in scalar context) containing references
+to hashes as its elements. Each hash contains the data for one menu item, and
+has the following key=value pairs:
 
  type        = The item type (e.g., 0, 1, I, g, etc.);
- text        = The item description (e.g., "A file you should download");
+ display     = The display string (e.g., "A file you should download");
  selector    = The selector string (e.g., /foo/bar);
  host        = The hostname (e.g., gopher.host.com);
  port        = The port number (e.g., 70);
  gopher_plus = The Gopher+ character (e.g., +, !, ?, etc.);
 
-The array will only contain hash refs of items that list some type of resource
-that can be downloaded; meaning that inline text ('i' item type) is skipped.
+Only items that list some type of resource that can be downloaded will be added
+to the list; meaning that inline text ('i' item type) is skipped.
 
 =cut
 
@@ -226,22 +232,41 @@ sub as_menu
 {
 	my $self = shift;
 
-	# get each item:
-	my @items = split(/$NEWLINE/, $self->{'content'});
+	# get the content, minus the period on a line by itself:
+	(my $content = $self->content) =~ s/\n\.\n?$//;
 
 	my @menu;
-	foreach my $item (@items)
+	MENU: foreach my $item (split(/\n/, $content))
 	{
-		# skip it if it's not a menu item:
-		next unless ($item =~ /\t/);
+		# get the item type and display string, selector, host, port,
+		# and Gopher+ string:
+		my ($type_and_display, $selector, $host, $port, $gopher_plus) =
+			split(/\t/, $item);
 
-		# create a hashref of this item:
-		my $item_hash = $self->_get_item_hashref($item);
+		# separate the item type and the item description:
+		my ($type, $display) = $type_and_display =~ /^(.)(.*)/;
 
 		# skip it if it's inline text:
-		next if ($item_hash->{'type'} eq 'i');
+		next if ($type eq 'i');
 
-		push(@menu, $item_hash);
+		foreach ($type, $display, $selector, $host, $port)
+		{
+			unless (defined $_)
+			{
+				carp "Couldn't parse menu item";
+				last MENU;
+			}
+		}
+
+		push(@menu, {
+				type        => $type,
+				display     => $display,
+				selector    => $selector,
+				host        => $host,
+				port        => $port,
+				gopher_plus => $gopher_plus
+			}
+		);
 	}
 
 	if (wantarray)
@@ -263,23 +288,29 @@ sub as_menu
 =head2 item_blocks([@block_names])
 
 C<item_blocks()>, C<directory_blocks()>, and C<as_blocks()> allow you to parse
-information blocks.
+information blocks. Each of these methods returns one or more information
+blocks in the form of L<Net::Gopher::Response::Blocks|BLOCK METHODS> objects.
 
-If the request was a Gopher+ item attribute information request, then you can
-use method to parse the attribute information blocks in the server's response.
 This method is a more simple alternative to the C<directory_blocks()> method.
 Use this method when you make item attribute information requests (!) and
 use C<directory_blocks()> when you make directory attribute information
 requests ($).
 
 This method can be used to retrieve item information block values by specifying
-the block name or block names as arguments. If you don't supply any block
-names, then this method will return a list containing every block name for the
-item.
+the block name or block names as arguments. When specifying names, remember
+that leading '+' and trailing ':' are stripped from block names, so rather
+than asking for '+INFO:', you should ask for just plain 'INFO'. If you don't
+supply any block names, then this method will return a list containing every
+block name for the item:
 
-Thess methods strips leading '+' and trailing ':' from block names, so rather
-than asking for '+INFO:', you should ask for just plain 'INFO'. These methods
-will also strip the leading spaces from each line of a multiline block value.
+ # Net::Gopher::Response::Blocks object for the INFO block:
+ my $info = $response->item_blocks('INFO');
+ 
+ # print the block value:
+ print $info->content;
+ 
+ # the name of every block:
+ my @block_names = $response->item_blocks;
 
 Please note that with this method, with the C<directory_blocks()> method, and
 with the C<as_blocks()> method, the blocks in the server's response are only
@@ -314,6 +345,10 @@ sub item_blocks
 
 =head2 directory_blocks([\%item | $item] [, @block_names])
 
+C<item_blocks()>, C<directory_blocks()>, and C<as_blocks()> allow you to parse
+information blocks. Each of these methods returns one or more information
+blocks in the form of L<Net::Gopher::Response::Blocks|BLOCK METHODS> objects.
+
 If the request was a Gopher+ directory attribute information request, then you
 can use method to get attribute information blocks for any of the items in the
 server's response. This method works like the C<item_blocks()> method,
@@ -328,14 +363,14 @@ The hash can contain any of the following key=value pairs:
 
  N          = The item must be the n'th item in the response.
  Type       = The item must be of this type.
- Text       = The item must have this description text.
+ Display    = The item must have this display string.
  Selector   = The item must have this selector string.
  Host       = The item must be on this host.
  Port       = The item must be at this port.
  GopherPlus = The item must have this Gopher+ string.
 
-So to get the VIEWS and ADMIN block values for the item with the selector of
-/welcome, you'd do this:
+So to get the VIEWS and ADMIN B<Net::Gopher::Response::Bocks> objects for the
+item with the selector of /welcome, you'd do this:
 
  my ($views, $admin) = $response->directory_blocks(
  	{Selector => '/welcome'}, 'VIEWS', 'ADMIN'
@@ -352,11 +387,12 @@ Or use even more options for more accuracy:
  	}, 'VIEWS'
  );
 
-Which means the VIEWS block value for the 7th item in the response, which must
-have a selector string of /welcome, on gopher.somehost.com at port 70.
+Which means the VIEWS B<Net::Gopher::Response::Blocks> object for the 7th item
+in the response, which must have a selector string of /welcome, on
+gopher.somehost.com at port 70.
 
 If you only want to specify the item by number, you can forgo the hash ref
-altogether. So get the ADMIN block for the second item, you can just do
+altogether. To get the ADMIN block object for the second item, you can just do
 this:
 
  my $admin = $response->directory_blocks(2, 'ADMIN');
@@ -364,7 +400,8 @@ this:
 To get the names of all of the information blocks for a single item, don't
 specify any block names, only a parameters hash or item number:
 
- my @block_names = $response->directory_blocks({
+ my @block_names = $response->directory_blocks(
+ 	{
  		Type     => 1,
  		Selector => '/stuff',
  		Host     => 'gopher.somehost.com',
@@ -382,29 +419,32 @@ or block names:
 
  my $num_items = $response->directory_blocks;
 
-This method will return undef if it couldn't find any of the blocks you
-specified, or if the item you specified does not exist.
+Please note that with this method, with the C<directory_blocks()> method, and
+with the C<as_blocks()> method, the blocks in the server's response are only
+parsed once, the first time you call any one of these methods, and stored in
+the response object, so multiple calls to any of these three methods will not
+result in performance degradation.
 
 =cut
 
 sub directory_blocks
 {
 	my $self        = shift;
-	my $item_to_get = shift;
+	my $from_item   = shift;
 	my @block_names = @_;
 
 	$self->_parse_blocks() unless (defined $self->{'blocks'});
 
-	if (defined $item_to_get and ref $item_to_get)
+	if (defined $from_item and ref $from_item)
 	{
 		my %match;
-		if (ref $item_to_get eq 'ARRAY')
+		if (ref $from_item eq 'ARRAY')
 		{
-			%match = @$item_to_get;
+			%match = @$from_item;
 		}
 		else
 		{
-			%match = %$item_to_get;
+			%match = %$from_item;
 		}
 
 		# a reference to hash contaiing the block names and values
@@ -416,59 +456,52 @@ sub directory_blocks
 				? $self->{'blocks'}[$match{'N'} - 1]
 				: @{ $self->{'blocks'} };
 
+		# the %match keys and corresponding as_info() keys:
+		my %keys = (
+			Type       => 'type',
+			Display    => 'display',
+			Selector   => 'selector',
+			Host       => 'host',
+			GopherPlus => 'gopher_plus'
+		);
+
 		# now search the items looking for the one that matches:
 		foreach my $item (@items)
 		{
-			my %info = $self->as_info_block($item->{'INFO'});
+			my %info = $item->{'INFO'}->as_info;
 
-			if (defined $match{'Type'})
+			my $matches = 1;
+			while (my ($m, $i) = each %keys)
 			{
-				unless ($info{'type'} eq $match{'Type'})
+				next unless (defined $match{$m});
+
+				if (ref $match{$m} eq 'Regexp')
 				{
-					next;
+					unless ($info{$i} =~ $match{$m})
+					{
+						$matches = 0;
+						last;
+					}
 				}
-			}
-			if (defined $match{'Text'})
-			{
-				unless ($info{'text'} eq $match{'Text'})
+				else
 				{
-					next;
-				}
-			}
-			if (defined $match{'Selector'})
-			{
-				unless ($info{'selector'} eq $match{'Selector'})
-				{
-					next;
-				}
-			}
-			if (defined $match{'Host'})
-			{
-				unless ($info{'host'} eq $match{'Host'})
-				{
-					next;
-				}
-			}
-			if (defined $match{'Port'})
-			{
-				unless ($info{'port'} eq $match{'Port'})
-				{
-					next;
-				}
-			}
-			if (defined $match{'GopherPlus'})
-			{
-				unless ($info{'gopher_plus'} eq $match{'GopherPlus'})
-				{
-					next;
+					unless ($info{$i} eq $match{$m})
+					{
+						$matches = 0;
+						last;
+					}
 				}
 			}
 
-			# it matches:
-			$matching_item = $item;
-			last;
+			if ($matches)
+			{
+				$matching_item = $item;
+				last;
+			}
 		}
 
+
+
 		return unless ($matching_item);
 
 		if (@block_names)
@@ -480,9 +513,9 @@ sub directory_blocks
 			return sort keys %$matching_item;
 		}
 	}
-	elsif (defined $item_to_get)
+	elsif (defined $from_item)
 	{
-		my $i = $item_to_get - 1;
+		my $i = $from_item - 1;
 
 		if (@block_names)
 		{
@@ -510,6 +543,10 @@ sub directory_blocks
 
 =head2 as_blocks()
 
+C<item_blocks()>, C<directory_blocks()>, and C<as_blocks()> allow you to parse
+information blocks. Each of these methods returns one or more information
+blocks in the form of L<Net::Gopher::Response::Blocks|BLOCK METHODS> objects.
+
 This method can be used to directly get all of the information blocks at once.
 If you made a directory attribute information request, then the blocks are
 stored in an array, where each element of the array is reference to a hash
@@ -520,13 +557,9 @@ to the array:
  my @items = $response->as_blocks;
  
  # INFO block for the second item:
- my %info = $response->as_info_block($items[1]{'INFO'});
+ my %info = $items[1]{'INFO'}->as_info;
 
- print "Type: $info{'type'}\n",
-       "Description: $info{'text'}\n",
-       "Selector: $info{'selector'}\n",
-       "On: $info{'host'}\n",
-       "Port: $info{'port'}\n";
+ print "$info{'display'} ($info{'host'}:$info{'port'}$info{'selector'})\n";
 
 If you made an item attribute information request, then the block
 names and values for the single item are stored in a hash, and the hash is
@@ -536,9 +569,15 @@ context:
  my %blocks = $response->as_blocks;
  
  # ADMIN block for the only item:
- my %admin = $response->as_admin_block($blocks{'ADMIN'});
+ my %admin = $blocks{'ADMIN'}->as_admin;
+ 
+ print "Run by $admin{'Admin'}[0] ($admin{'Admin'}[1]).\n";
 
- print "Run by $admin{'Admin'}[0] (contact: $admin{'Admin'}[1]).\n";
+Please note that with this method, with the C<directory_blocks()> method, and
+with the C<as_blocks()> method, the blocks in the server's response are only
+parsed once, the first time you call any one of these methods, and stored in
+the response object, so multiple calls to any of these three methods will not
+result in performance degradation.
 
 =cut
 
@@ -548,369 +587,19 @@ sub as_blocks
 
 	$self->_parse_blocks() unless (defined $self->{'blocks'});
 
-	# Since $self->{'blocks'} contains an array of references to hashes, we
-	# can't just return one or more of those reference because then when
-	# the user tried to modify any of the item blocks they received, they'd
-	# end up modifying the copies stored in $self->{'blocks'} too. So we'll
-	# create a copy of $self->{'blocks'} with references to anonymous
-	# hashes containing the same values as those in $self->{'blocks'}:
-	my @blocks;
+	my @items;
 	foreach my $item (@{ $self->{'blocks'} })
 	{
-		push(@blocks, { %$item });
+		push(@items, { %$item });
 	}
 
-	if (@blocks == 1)
+	if (@items == 1)
 	{
-		if (wantarray)
-		{
-			return %{ $blocks[0] };
-		}
-		else
-		{
-			return $blocks[0];
-		}
+		return wantarray ? %{ $items[0] } : $items[0];
 	}
 	else
 	{
-		if (wantarray)
-		{
-			return @blocks;
-		}
-		else
-		{
-			\@blocks;
-		}
-	}
-}
-
-
-
-
-
-#==============================================================================#
-
-=head2 as_admin_block([$block_value])
-
-ADMIN blocks contain information about the person running the Gopher+ server
-and what the admin has done with the item in question. This method takes a
-string containing an ADMIN block value as its only argument, parses the ADMIN
-block, and returns a hash (in list context) or a reference to a hash (in scalar
-context) with (at least) two key=value pairs: Admin and Mod-Date.
-
-Admin attributes contain strings in the form of "John Doe <jdoe@fake.email>"
-about who the administrator of the Gopher+ server is. This method will parse
-Admin attributes and turn them into a reference to a two-element array
-containing the name and email in that order.
-
-Mod-Date is a timestamp of when the item was last modified. This method will
-convert the timestamp into an array containing values in the same format as
-those returned by Perl's C<localtime()> function corresponding with the
-Mod-Date timestamp (to find out exactly what the array will contain, see
-C<perldoc -f localtime>):
-
- my %admin = $response->as_admin_block($response->item_blocks('ADMIN'));
- 
- my ($name, $email) = @{ $admin{'Admin'} }
- print "This box is maintained by $name ($email).";
- 
- my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) =
- @{ $admin{'Mod-Date'} };
-
-The hash may also contain Abstract, Version, Org, Loc or other attributes--all
-of which are stored as in the hash as plain text.
-
-If you don't supply a block value, then this method will call
-item_blocks('ADMIN') and try to use that instead.
-
-=cut
-
-sub as_admin_block
-{
-	my $self  = shift;
-	my $block = shift;
-
-	$block = $self->item_blocks('ADMIN') unless (defined $block);
-
-	# ADMIN blocks contain two attributes, Admin and
-	# Mod-Date, in the form of:
-	# 
-	#   Admin: Foo Bar <foobar@foo.com>
-	#   Mod-Date: WWW MMM DD hh:mm:ss YYYY <YYYYMMDDhhmmss>
-	# 
-	# first, get the Admin, Mod-Date, and any other
-	# attributes in the form of a hash:
-	my %attributes;
-	foreach my $attribute (split(/$NEWLINE/, $block))
-	{
-		# get the "Name: value" attribute:
-		my ($name, $value) = $attribute =~ /^(.+?):\s?(.*)/;
-
-		$attributes{$name} = $value;
-	}
-
-
-	if (exists $attributes{'Admin'})
-	{
-		# now for the Admin attribute, get the admin name and email:
-		my ($name, $email) =
-			$attributes{'Admin'} =~ /(.+?)\s*<(.*?)>\s*/;
-
-
-		$attributes{'Admin'} = [$name, $email];
-	}
-
-	if (exists $attributes{'Mod-Date'})
-	{
-		# now for the Mod-Date attribute, get the values from the
-		# timestamp:
-		my ($year, $month, $day, $hour, $minute, $second) =
-			$attributes{'Mod-Date'} =~
-				/<(\d{4})(\d{2})(\d{2})(\d{2})(\d{2})(\d{2})>/x;
-
-		foreach($year, $month, $day, $hour, $minute, $second)
-		{
-			carp "Couldn't parse timestamp" unless (defined $_);
-		}
-
-		# We need to convert the year value into the  number years
-		# since 1900 (i.e., 2003 -> 103), since that's the format
-		# returned by localtime():
-		$year -= 1900;
-
-		# localtime() months are numbered from 0 to 11, not 1 to 12:
-		$month--;
-
-		# now that we have the second, minute, hour, day, month, and
-		# year, we use them to get a corresponding time() value:
-		my $time = timelocal(
-			$second, $minute, $hour, $day, $month, $year
-		);
-
-		# now use the time() value to get the values we still don't
-		# have (e.g., the day of the year, is it daylight savings time?
-		# etc.) and store them in the Mod-Date attribute value:
-		$attributes{'Mod-Date'} = [ localtime $time ];
-	}
-
-
-
-	if (wantarray)
-	{
-		return %attributes;
-	}
-	else
-	{
-		return \%attributes;
-	}
-}
-
-
-
-
-
-#==============================================================================#
-
-=head2 as_ask_block([$block_value])
-
-ASK blocks contain a form to be filled out by the user, with ASK queries on
-lines by themselves consisting of query type, followed by the question and any
-default values separated by tabs (e.g., "Ask: Some question?\tdefault
-answer 1\tdefault answer 2", "Choose: A question?choice 1\tchoice 2\tchoice3").
-
-This method takes an ASK block in a string as its only argument and parses the
-ASK block and will return an array (in list context) or a reference to an
-array (in scalar context) containing hash refs of each query in the order
-they appeared, with each hash having the following key=value pairs:
-
- type     = The type of query (e.g, Ask, AskP, Select, Choose, etc.).
- question = The question.
- defaults = A reference to an array containing the default answers.
-
-If you don't supply a block value, then this method will call
-item_blocks('ASK') and try to use that instead.
-
-=cut
-
-sub as_ask_block
-{
-	my $self  = shift;
-	my $block = shift;
-
-	$block = $self->item_blocks('ASK') unless (defined $block);
-
-
-
-	# ASK blocks contain Gopher+ queries which are to be filled out by the
-	# user. Each ASK query has a type, followed by a quetion and optional
-	# defaults separated by tabs. For example:
-	#    Ask: How many?\tone\ttwo\tthree
-	#    AskP: Your password:
-	#    Choose: Pick one:\tred\tgreen\tblue
-	#    
-	# This will store each ASK query as a hashref containing the query
-	# type, the question and any defaults:
-	my @ask;
-
-	foreach my $query (split(/$NEWLINE/, $block))
-	{
-		# get the query type, and the question and default values:
-		my ($type, $question_and_defaults) = $query =~ /^(\S+)+:\s?(.*)/;
-
-		# the question and any default values are all tab separated:
-		my ($question, @defaults) = split(/\t/, $question_and_defaults);
-
-		push(@ask, {
-				type     => $type,
-				question => $question,
-				defaults => (@defaults)
-						? \@defaults
-						: undef
-			}
-		);	
-	}
-
-
-
-	if (wantarray)
-	{
-		return @ask;
-	}
-	else 
-	{
-		return \@ask;
-	}
-}
-
-
-
-
-
-#==============================================================================#
-
-=head2 as_info_block([$block_value])
-
-This method takes an INFO block value as its only argument, parses it, and
-returns a hash (in list context) or a reference to a hash (in scalar context)
-containing the information from the tab delimited fields in the same format
-described above (see L<as_menu()|Net::Gopher::Response/as_menu()>). So you can
-use it like this:
-
- my %info = $response->as_info_block($response->item_blocks('INFO'));
- 
- print "Item type: $info{'type'};\n",
-       "Item description: $info{'text'};\n",
-       "Selector: $info{'selector'};\n",
-       "On: $info{'host'};\n",
-       "At: $info{'port'};\n";
-
-If you don't supply a block value, then this method will call
-item_blocks('INFO') and try to use that instead.
-
-=cut
-
-sub as_info_block
-{
-	my $self  = shift;
-	my $block = shift;
-
-	$block = $self->item_blocks('INFO') unless (defined $block);
-
-
-
-	my $info = $self->_get_item_hashref($block);
-
-	if (wantarray)
-	{
-		return %$info;
-	}
-	else
-	{
-		return $info;
-	}
-}
-
-
-
-
-
-#==============================================================================#
-
-=head2 as_views_block([$block_value])
-
-VIEWS blocks contain information about what type of applications can be used
-to view the item, the total size of the item, and sometimes a language as well
-(e.g., text/plain En_US: <77K>).
-
-This method takes a string containing a VIEWS block as its only argument,
-parses it, and returns an array (in list context) or a reference to an array
-(in scalar context) containing each view in the form of a hash ref with the the
-following key=value pairs:
-
- type     = The MIME type (e.g., text/plain, application/gopher+-menu, etc.).
- language = The ISO 639 language code (e.g., En_US).
- size     = The size in bytes.
-
-Note that this method will convert the <\d+K?> size format used in Gopher+ to
-an integer; the total size in bytes (e.g., <80> becomes 80, <40K> becomes
-40000, <.4K> becomes 400, <400B> becomes 400, etc.):
-
- my @views = $response->as_views_block($response->item_blocks('VIEWS'));
- 
- foreach my $view (@views) {
- 	print "$view->{'type'} ($view->{'size'} bytes) ($type->{'language'})\n";
- }
-
-If you don't supply a block value, then this method will call
-item_blocks('VIEWS') and try to use that instead.
-
-=cut
-
-sub as_views_block
-{
-	my $self  = shift;
-	my $block = shift;
-
-	$block = $self->item_blocks('VIEWS') unless (defined $block);
-
-
-
-	# Views blocks contain attributes in the form of
-	# "MIME-type lang: <size>" (e.g., "text/plain En_US: <10K>."). This
-	# array will store each view as a hashref:
-	my @views;
-
-	foreach my $view (split(/$NEWLINE/, $block))
-	{
-		# separate the MIME type, language, and size:
-		my ($mime_type, $lang, $size) =
-			$view =~ /^([^:]*?) (?: \s ([^:]{5}) )?:(.*)$/x;
-
-		if (defined $size and $size =~ /<(\.?\d+)(?:(k)|b)?>/i)
-		{
-			# turn <55> into 55, <600B> into 600, <55K> into 55000,
-			# and <.5K> into 500:
-			$size  = $1;
-			$size *= 1000 if ($2);
-		}
-
-		push(@views, {
-				type     => $mime_type,
-				language => $lang,
-				size     => $size
-			}
-		);
-	}
-
-
-
-	if (wantarray)
-	{
-		return @views;
-	}
-	else
-	{
-		return \@views;
+		return wantarray ? @items : \@items;
 	}
 }
 
@@ -923,7 +612,7 @@ sub as_views_block
 =head2 is_success()
 
 This method will return true if the request was successful, false otherwise.
-First, weather it's a Gopher or Gopher+ request, it won't be "successful" if
+First, whether it's a Gopher or Gopher+ request, it won't be "successful" if
 any network errors occurred. Beyond that, in Gopher+, for a request to be a
 "success" means that the status code returned by the server indicated success
 (a code of +). In plain old Gopher, success is rather loosely defined.
@@ -942,9 +631,9 @@ sub is_success
 {
 	my $self = shift;
 
-	if (defined $self->{'status'})
+	if (defined $self->status)
 	{
-		if ($self->{'status'} eq '+')
+		if ($self->status eq '+')
 		{
 			return 1;
 		}
@@ -953,7 +642,7 @@ sub is_success
 			return;
 		}
 	}
-	elsif (defined $self->{'error'})
+	elsif (defined $self->error)
 	{
 		return;
 	}
@@ -981,9 +670,9 @@ sub is_error
 {
 	my $self = shift;
 
-	if (defined $self->{'status'})
+	if (defined $self->status)
 	{
-		if ($self->{'status'} eq '-')
+		if ($self->status eq '-')
 		{
 			return 1;
 		}
@@ -992,7 +681,7 @@ sub is_error
 			return;
 		}
 	}
-	elsif (defined $self->{'error'})
+	elsif (defined $self->error)
 	{
 		return 1;
 	}
@@ -1019,7 +708,9 @@ sub is_blocks
 {
 	my $self = shift;
 
-	if ($self->{'content'} =~ /^(?:\+\S+\s.*?) (?:$NEWLINE\+\S+ \s .*?)*$/sx)
+	my $block = qr/\+\S+ \s .*?/sx;
+
+	if ($self->content =~ /^$block (?: \n$block)*$/sx)
 	{
 		return 1;
 	}
@@ -1049,9 +740,7 @@ sub is_menu
 	my $field = qr/[^\t\012\015]*?/;
 	my $item  = qr/$field\t$field\t$field\t$field (?:\t[\+\!\?\$])?/x;
 
-	if ($self->{'content'} =~
-		/^$item (?:$NEWLINE $item)*
-		 (?:$NEWLINE\.$NEWLINE?|$NEWLINE)? $/x)
+	if ($self->content =~ /^ $item (?:\n $item)* (?:\n\.\n?|\n)? $/x)
 	{
 		return 1;
 	}
@@ -1079,7 +768,11 @@ sub is_terminated
 	my $self  = shift;
 	my $error = shift;
 
-	if ($self->{'content'} =~ /$NEWLINE\.$NEWLINE?$/)
+	# Since as_string() returns the unmodified response, it will always
+	# have the period on a line by itself in it; but that also means the
+	# line endings weren't converted to LF, so we can't use \n to match the
+	# period on a line by itself:
+	if ($self->as_string =~ /$NEWLINE\.$NEWLINE?$/)
 	{
 		return 1;
 	}
@@ -1108,56 +801,6 @@ sub error { return shift->{'error'} }
 
 
 
-################################################################################
-#
-#	Method
-#		_get_item_hashref($item)
-#
-#	Purpose
-#		This method parses an item in a Gopher menu and extracts the
-#		item type, item description, selector string, host, port, and
-#		Gopher+ character. It returns a reference to hash with the
-#		following keys: "type"; "text"; "selector"; "host"; "port"; and
-#		"gopher_plus".
-#
-#	Parameters
-#		$item - A string containing the menu item.
-#
-
-sub _get_item_hashref
-{
-	my $self = shift;
-	my $item = shift;
-
-	# get the item type and description text, selector, host, port, and
-	# Gopher+ string:
-	my ($type_and_text, $selector, $host, $port, $gopher_plus) =
-		split(/\t/, $item);
-
-	# now we need to separate the item type and the item description:
-	my ($type, $text) = $type_and_text =~ /^(.)(.*)/;
-
-	foreach ($type, $text, $selector, $host, $port)
-	{
-		carp "Couldn't parse menu item" unless (defined $_);
-	}
-
-	my $item_hash = {
-		type        => $type,
-		text        => $text,
-		selector    => $selector,
-		host        => $host,
-		port        => $port,
-		gopher_plus => $gopher_plus
-	};
-
-	return $item_hash;
-}
-
-
-
-
-
 ################################################################################
 #
 #	Method
@@ -1177,7 +820,6 @@ sub _get_item_hashref
 sub _parse_blocks
 {
 	my $self    = shift;
-	my $content = $self->{'content'};
 
 	# $self->{'blocks'} will contain a reference to an array which
 	# will have hashrefs as its elements. Each hash will contain the item
@@ -1189,23 +831,15 @@ sub _parse_blocks
 	# elements:
 	$self->{'blocks'} = [];
 
-	# Each block name is denoted by '+' as the first character at the start
-	# of line. Any characters after the plus and up to the first space is
-	# the block name, and everything after the first space is the value.
-	if ($self->is_terminated)
-	{
-		$content =~ s/$NEWLINE\.$NEWLINE?$//;
-	}
-
 	# remove all leading whitespace and the leading + for the first block
 	# name:
-	$content =~ s/^\s*\+//;
+	(my $content = $self->content) =~ s/^\s*\+//;
 
 	# this will store the block names and block values for each item, one
 	# at a time:
 	my %blocks;
 
-	foreach my $name_and_value (split(/$NEWLINE\+/, $content))
+	foreach my $name_and_value (split(/\n\+/, $content))
 	{
 		# get the space separated name and value:
 		my ($name, $value) = $name_and_value =~ /(\S+)\s(.*)/s;
@@ -1213,14 +847,6 @@ sub _parse_blocks
 		# block names are usually postfixed with colons:
 		$name =~ s/:$//;
 
-		# now remove the leading spaces from each attribute:
-		$value =~ s/($NEWLINE)\s/$1/g;
-
-		# block values with multiple attributes often start with the
-		# first attribute on the next line with a leading space, so
-		# we remove the leading newline:
-		$value =~ s/^$NEWLINE//;
-
 		# if the current item already has a block by this name, then
 		# this block belongs to the next item, so save this item's
 		# hash and start a new one:
@@ -1237,10 +863,13 @@ sub _parse_blocks
 
 
 
-		$blocks{$name} = $value;
+		$blocks{$name} = new Net::Gopher::Response::Blocks (
+			BlockName  => $name,
+			BlockValue => $value
+		);
 	}
 
-	# add the last item's attribute information blocks:
+	# add the last item's attribute information blocks to the list:
 	push(@{ $self->{'blocks'} }, { %blocks });
 }
 
@@ -1248,6 +877,183 @@ sub _parse_blocks
 
 __END__
 
+
+=head1 BLOCK METHODS
+
+The C<item_blocks()>, C<directory_blocks()>, and C<as_blocks()> methods all
+return one or more information blocks in the form of
+B<Net::Gopher::Response::Blocks> objects. Here are all of the methods you can
+call on these objects:
+
+=head2 content()
+
+This method will return the content of the block value, with the leading space
+at the beginning of each line removed.
+
+=head2 as_string()
+
+This method will return the unmodified block value, with leading spaces still
+intact.
+
+=head2 as_admin()
+
+ADMIN blocks contain attributes detailing information about a particular item
+including who the administrator of it is and when it was last modified. ADMIN
+blocks have at least two attributes: I<Admin> and I<Mod-Date>, though they can
+(and often do) contain many more.
+
+The I<Admin> attribute contains the name of the administrator and his or her
+email address (e.g., "John Doe <jdoe@notreal.email>"). I<Mod-Date> contains a
+timestamp for when the item was last modified.
+
+Like C<as_attributes()>, this method will parse these attributes and return a
+hash (in list context) or a reference to a hash (in scalar context) containing
+the attribute names and values. Unlike C<as_attributes()>, this method parses
+the individual I<Admin> and I<Mod-Date> attributes values and stores them as
+arrays (each described below).
+
+For the I<Admin> attribute, this method will extract the administrator name and
+email from the Admin attribute value and create a two element array with them:
+
+ my %admin = $response->item_blocks('ADMIN')->as_admin;
+ 
+ my ($name, $email) = @{ $admin{'Admin'} };
+ print "Run by $name ($email).n";
+
+For the I<Mod-Date> attribute, this method will extract the timestamp convert
+it into an array containing values in the same format as those returned by
+Perl's C<localtime()> function corresponding with the timestamp (to find out
+exactly what the array will contain, see C<perldoc -f localtime>):
+
+ my %admin = $response->item_blocks('ADMIN')->as_admin;
+ 
+ my ($name, $email) = @{ $admin{'Admin'} }
+ print "This box is maintained by $name ($email).";
+ 
+ my ($sec, $min, $hour, $mday, $mon, $year, $wday, $yday, $isdst) =
+ @{ $admin{'Mod-Date'} };
+
+As mentioned above, in addition to I<Admin> and I<Mod-Date>, the hash may
+contain other attributes including I<Abstract>, I<Version>, I<Org>, I<Loc> and
+others--all of which are stored as in the hash as plain text, not as arrays.
+
+Note that this method is inherited by B<Net::Gopher::Response>. You can call
+this method directly on a B<Net::Gopher::Response> object, in which case
+this method will call C<$response-E<gt>item_blocks('ADMIN')> and use that.
+Thus, this:
+
+ my %admin = $response->as_admin;
+
+is the same as this:
+
+ my %admin = $response->item_blocks('ADMIN')->as_admin;
+
+=head2 as_ask()
+
+ASK blocks contain a form to be filled out by the user, with ASK queries on
+lines by themselves consisting of query type, followed by the question and any
+default values separated by tabs (e.g., "Ask: Some question?\tdefault
+answer 1\tdefault answer 2", "Choose: A question?choice 1\tchoice 2\tchoice3").
+
+This method parses the ASK block and will return an array (in list context) or
+a reference to an array (in scalar context) containing hash refs of each query
+in the order they appeared, with each hash having the following key=value
+pairs:
+
+ type     = The type of query (e.g, Ask, AskP, Select, Choose, etc.).
+ question = The question.
+ defaults = A reference to an array containing the default answers.
+
+Note that this method is inherited by B<Net::Gopher::Response>. You can call
+this method directly on a B<Net::Gopher::Response> object, in which case
+this method will call C<$response-E<gt>item_blocks('ASK')> and use that.
+Thus, this:
+
+ my @ask = $response->as_ask;
+
+is the same as this:
+
+ my @ask = $response->item_blocks('ASK')->as_ask;
+
+=head2 as_info()
+
+INFO blocks contain tab delimited item information like that which you'd find
+in a Gopher menu.
+
+This method parses INFO blocks and returns a hash (in list context) or a
+reference to a hash (in scalar context) containing the information from the
+tab delimited fields in the same format as described above
+(see L<as_menu()|Net::Gopher::Response/as_menu()>):
+
+ my %info = $response->item_blocks('INFO')->as_info;
+ 
+ $gopher->connect($info{'host'}, Port => $info{'port'}) or die $gopher->error;
+ 
+ my $another_response = $gopher->request($info{'selector'});
+
+Note that this method is inherited by B<Net::Gopher::Response>. You can call
+this method directly on a B<Net::Gopher::Response> object, in which case
+this method will call C<$response-E<gt>item_blocks('INFO')> and use that.
+Thus, this:
+
+ my %info = $response->as_info;
+
+is the same as this:
+
+ my %info = $response->item_blocks('INFO')->as_info;
+
+=head2 as_views()
+
+VIEWS blocks contain a list of available formats for a particular item.
+
+This method parses VIEWS blocks and returns an array (in list context) or a
+reference to an array (in scalar context) containing each view in the form of a
+reference to hash with the following key=value pairs:
+
+ type     = The MIME type (e.g., text/plain, application/gopher+-menu, etc.).
+ language = The ISO 639 language code (e.g., En_US).
+ size     = The size in bytes.
+
+Note that this method will convert the <> size format used in Gopher+ to
+an integer; the total size in bytes (e.g., <80> becomes 80, <40K> becomes
+40000, <.4K> becomes 400, <400B> becomes 400, etc.):
+
+ my @views = $response->item_blocks('VIEWS')->as_views;
+ 
+ foreach my $view (@views) {
+ 	print "$view->{'type'} ($view->{'size'} bytes) ($type->{'language'})\n";
+ ...
+ 	my $another_response = $gopher->request(
+ 		$selector,
+ 		Representation => $view->{'type'}
+ 	);
+ ...
+ }
+
+Note that this method is inherited by B<Net::Gopher::Response>. You can call
+this method directly on a B<Net::Gopher::Response> object, in which case
+this method will call C<$response-E<gt>item_blocks('VIEWS')> and use that.
+Thus, this:
+
+ my @views = $response->as_views;
+
+is the same as this:
+
+ my @views = $response->item_blocks('VIEWS')->as_views;
+
+=head2 as_attributes()
+
+If the block value contains a series of C<Name: value> attributes on lines by
+themselves, then you can use this method to parse them. This method will
+return a hash (in list context) or a reference to a hash (in scalar context)
+containing the attribute names and values.
+
+=head2 is_attributes()
+
+This method checks to see if the block value can be successfully parsed by
+C<as_attributes()>. If the value contains a series of C<Name: value> attributes,
+then this method will return true; false otherwise.
+
 =head1 BUGS
 
 If you encounter bugs, you can alert me of them by emailing me at
